@@ -1,11 +1,11 @@
 #include <cstdio>
 #include <cstdlib>
 
-#include <boost/spirit/include/qi_parse_attr.hpp>
 #include <boost/spirit/include/qi_numeric.hpp>
+#include <boost/spirit/include/qi_parse_attr.hpp>
 
-#include "MaeParser.hpp"
 #include "MaeBlock.hpp"
+#include "MaeParser.hpp"
 
 #define WHITESPACE ' ' : case '\n' : case '\r' : case '\t'
 
@@ -21,16 +21,16 @@ static bool property_key_author_name(Buffer& buffer, char*& save);
 
 static std::string outer_block_name(Buffer& buffer);
 
-void read_exception::format(int line_number, int column, const char* msg)
+void read_exception::format(size_t line_number, size_t column, const char* msg)
 {
 #ifdef _MSC_VER
     _snprintf(m_msg, MAEPARSER_EXCEPTION_BUFFER_SIZE,
-              "Line %d, column %d: %s\n", line_number,
+              "Line %Iu, column %Iu: %s\n",
 #else
-    snprintf(m_msg, MAEPARSER_EXCEPTION_BUFFER_SIZE, "Line %d, column %d: %s\n",
-             line_number,
+    snprintf(m_msg, MAEPARSER_EXCEPTION_BUFFER_SIZE,
+             "Line %zu, column %zu: %s\n",
 #endif
-              column, msg);
+              line_number, column, msg);
     m_msg[MAEPARSER_EXCEPTION_BUFFER_SIZE - 1] = '\0';
 }
 
@@ -88,6 +88,18 @@ bool character(char c, Buffer& buffer, char*& save)
     }
 }
 
+static void remove_escape_characters(std::string& s)
+{
+    size_t j = 0;
+    for (size_t i = 0; i < s.size(); ++i, ++j) {
+        if (s[i] == '\\')
+            ++i;
+        if (j < i)
+            s[j] = s[i];
+    }
+    s.resize(j);
+}
+
 /**
  * Read an integer and return its value. An integer is terminated
  * either by whitespace or a ']'.
@@ -192,10 +204,13 @@ EXPORT_MAEPARSER std::string parse_value<std::string>(Buffer& buffer)
         return std::string(save, buffer.current);
     } else {
         save = ++buffer.current;
+        std::string rval;
         while (buffer.current < buffer.end || buffer.load(save)) {
             switch (*buffer.current) {
             case '"':
-                return std::string(save, buffer.current++);
+                rval = std::string(save, buffer.current++);
+                remove_escape_characters(rval);
+                return rval;
             case '\\':
                 ++buffer.current;
                 break;
@@ -206,7 +221,8 @@ EXPORT_MAEPARSER std::string parse_value<std::string>(Buffer& buffer)
     }
 }
 
-template <> EXPORT_MAEPARSER bool parse_value<bool>(Buffer& buffer)
+template <>
+EXPORT_MAEPARSER BoolProperty parse_value<BoolProperty>(Buffer& buffer)
 {
     bool value = false;
     if (*buffer.current == '1') {
@@ -310,57 +326,56 @@ std::string MaeParser::blockBeginning(int* indexed)
 std::shared_ptr<Block> MaeParser::blockBody(const std::string& name)
 {
     auto block = std::make_shared<Block>(name);
-    IndexedBlockParser* indexed_block_parser = getIndexedBlockParser();
+    auto indexed_block_parser =
+        std::shared_ptr<IndexedBlockParser>(getIndexedBlockParser());
 
     std::vector<std::shared_ptr<std::string>> property_names;
     schrodinger::mae::whitespace(m_buffer);
     properties(&property_names);
 
-    std::vector<std::shared_ptr<std::string>>::const_iterator iter =
-        property_names.begin();
-    std::string s;
-    for (; iter != property_names.end(); ++iter) {
+    for (auto& property_name : property_names) {
         schrodinger::mae::whitespace(m_buffer);
-        switch ((*(*iter))[0]) {
+        switch ((*property_name)[0]) {
         case 'r':
-            block->setRealProperty((*(*iter)), parse_value<double>(m_buffer));
+            block->setRealProperty(*property_name,
+                                   parse_value<double>(m_buffer));
             break;
         case 's':
-            block->setStringProperty((*(*iter)),
+            block->setStringProperty(*property_name,
                                      parse_value<std::string>(m_buffer));
             break;
         case 'i':
-            block->setIntProperty((*(*iter)), parse_value<int>(m_buffer));
+            block->setIntProperty(*property_name, parse_value<int>(m_buffer));
             break;
         case 'b':
-            block->setBoolProperty((*(*iter)), parse_value<bool>(m_buffer));
+            block->setBoolProperty(*property_name,
+                                   1u == parse_value<BoolProperty>(m_buffer));
             break;
         }
     }
-    schrodinger::mae::whitespace(m_buffer);
-    while (true) {
+
+    auto advance = [this]() {
+        schrodinger::mae::whitespace(m_buffer);
         if (!m_buffer.load()) {
             throw read_exception(m_buffer, "Missing '}' for block.");
         }
+    };
 
-        if (*m_buffer.current == '}') {
-            ++m_buffer.current;
-            break;
-        }
-
-        int indexed = 0;
+    int indexed = 0;
+    for (advance(); *m_buffer.current != '}'; advance()) {
         std::string name = blockBeginning(&indexed);
         if (indexed) {
             indexed_block_parser->parse(name, indexed, m_buffer);
         } else {
-            std::shared_ptr<Block> sub_block =
-                std::shared_ptr<Block>(blockBody(name));
+            std::shared_ptr<Block> sub_block = blockBody(name);
             block->addBlock(sub_block);
         }
-        schrodinger::mae::whitespace(m_buffer);
     }
-    block->setIndexedBlockMap(std::shared_ptr<IndexedBlockMapI>(
-        indexed_block_parser->getIndexedBlockMap()));
+
+    ++m_buffer.current;
+
+    block->setIndexedBlockMap(indexed_block_parser->getIndexedBlockMap());
+
     return block;
 }
 
@@ -369,7 +384,7 @@ void MaeParser::properties(
 {
     std::shared_ptr<std::string> property_name;
     while ((property_name = property_key(m_buffer)) != nullptr) {
-        property_names->push_back(std::shared_ptr<std::string>(property_name));
+        property_names->push_back(property_name);
         schrodinger::mae::whitespace(m_buffer);
     }
     triple_colon(m_buffer);
@@ -502,10 +517,6 @@ void IndexedBlockBuffer::value(Buffer& buffer)
     }
 }
 
-DirectIndexedBlockParser::~DirectIndexedBlockParser()
-{
-}
-
 void DirectIndexedBlockParser::parse(const std::string& name, size_t size,
                                      Buffer& buffer)
 {
@@ -531,7 +542,7 @@ void DirectIndexedBlockParser::parse(const std::string& name, size_t size,
     for (auto& key : property_keys) {
         switch (key[0]) {
         case 'b':
-            p = new IndexedValueCollector<bool>(key, size);
+            p = new IndexedValueCollector<BoolProperty>(key, size);
             break;
         case 'i':
             p = new IndexedValueCollector<int>(key, size);
@@ -663,7 +674,7 @@ IndexedBlock* IndexedBlockBuffer::getIndexedBlock()
         size_t len;
         switch (type) {
         case 'b': {
-            std::vector<bool> bvalues;
+            std::vector<BoolProperty> bvalues;
             bvalues.reserve(m_rows);
             for (size_t ix = prop_indx; ix < value_count; ix += col_count) {
                 getData(ix, &data, &len);
@@ -743,7 +754,13 @@ IndexedBlock* IndexedBlockBuffer::getIndexedBlock()
                     is_null->set(svalues.size());
                     svalues.emplace_back();
                 } else {
-                    svalues.emplace_back(data, len);
+                    if (data[0] != '"') { // Check for quote wrapping
+                        svalues.emplace_back(data, len);
+                    } else { // During parsing we check for full quote wrapping
+                        auto rval = std::string(data + 1, len - 2);
+                        remove_escape_characters(rval);
+                        svalues.emplace_back(rval);
+                    }
                 }
             }
             std::shared_ptr<IndexedStringProperty> isp(
@@ -760,10 +777,6 @@ BufferedIndexedBlockParser::BufferedIndexedBlockParser()
     m_indexed_block_map = std::make_shared<BufferedIndexedBlockMap>();
 }
 
-BufferedIndexedBlockParser::~BufferedIndexedBlockParser()
-{
-}
-
 std::shared_ptr<IndexedBlockMapI>
 BufferedIndexedBlockParser::getIndexedBlockMap()
 {
